# Generated by Django 5.2.7 on 2025-10-13 17:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cart', '0010_cart_cart_created_at_ix_cart_cart_updated_at_ix_and_more'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='cart',
            name='unique_session_cart',
        ),
        migrations.RemoveIndex(
            model_name='cart',
            name='cart_session_key_ix',
        ),
        migrations.RemoveField(
            model_name='cart',
            name='session_key',
        ),
    ]
