# Generated by Django 6.0.3 on 2026-03-17 22:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0024_add_payment_method_fee'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='is_deleted',
            field=models.BooleanField(db_index=True, default=False),
        ),
    ]
