# Generated by Django 5.2.9 on 2026-01-17 16:23

import core.fields.image
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pay_way', '0010_alter_payway_icon'),
    ]

    operations = [
        migrations.AlterField(
            model_name='payway',
            name='icon',
            field=core.fields.image.ImageAndSvgField(blank=True, null=True, upload_to='uploads/pay_way/', verbose_name='Icon'),
        ),
    ]
