# Generated by Django 5.2.9 on 2026-01-24 20:14

import djmoney.models.fields
from decimal import Decimal
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0023_remove_mobile_phone_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='payment_method_fee',
            field=djmoney.models.fields.MoneyField(decimal_places=2, default=Decimal('0'), help_text='Fee charged by the payment method (e.g., Cash on Delivery fee)', max_digits=11, verbose_name='Payment Method Fee'),
        ),
        migrations.AddField(
            model_name='order',
            name='payment_method_fee_currency',
            field=djmoney.models.fields.CurrencyField(choices=[('EUR', 'EUR €'), ('USD', 'USD $')], default='EUR', editable=False, max_length=3),
        ),
    ]
