# Generated by Django 6.0.2 on 2026-03-01 12:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pay_way', '0011_alter_payway_icon'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paywaytranslation',
            name='name',
            field=models.CharField(blank=True, choices=[('CREDIT_CARD', 'Credit Card'), ('PAY_ON_DELIVERY', 'Pay On Delivery'), ('PAY_ON_STORE', 'Pay On Store'), ('PAY_PAL', 'PayPal'), ('STRIPE', 'Stripe'), ('BANK_TRANSFER', 'Bank Transfer'), ('APPLE_PAY', 'Apple Pay'), ('GOOGLE_PAY', 'Google Pay'), ('VIVA_WALLET', 'Viva Wallet')], max_length=50, null=True, verbose_name='Name'),
        ),
    ]
