# Generated by Django 5.2.9 on 2026-01-24 13:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0022_make_payment_id_nullable'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='order',
            name='mobile_phone',
        ),
    ]
