# Generated by Django 5.2.9 on 2026-01-23 19:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0021_add_stock_reservation_ids_to_order'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='payment_id',
            field=models.CharField(blank=True, default='', max_length=255, null=True, verbose_name='Payment ID'),
        ),
    ]
