# Generated by Django 6.0.3 on 2026-03-17 22:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0028_historicalproduct_points_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalproduct',
            name='is_deleted',
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name='product',
            name='is_deleted',
            field=models.BooleanField(db_index=True, default=False),
        ),
    ]
