# Generated by Django 5.2.9 on 2026-01-21 16:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0019_add_stock_log_model'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='stock_reservation_ids',
            field=models.JSONField(blank=True, default=list, help_text='List of stock reservation IDs that were converted to this order. Provides audit trail linking reservations to final orders.', verbose_name='Stock Reservation IDs'),
        ),
    ]
