# Generated by Django 5.2.9 on 2026-01-22 15:51

import djmoney.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cart', '0011_remove_session_key'),
    ]

    operations = [
        migrations.AddField(
            model_name='cartitem',
            name='price_at_add',
            field=djmoney.models.fields.MoneyField(blank=True, decimal_places=2, help_text='Price of the product when added to cart (for price change validation)', max_digits=10, null=True, verbose_name='Price at Add'),
        ),
        migrations.AddField(
            model_name='cartitem',
            name='price_at_add_currency',
            field=djmoney.models.fields.CurrencyField(choices=[('EUR', 'EUR €'), ('USD', 'USD $')], default='EUR', editable=False, max_length=3, null=True),
        ),
    ]
