# Generated by Django 5.2.9 on 2026-01-24 13:28

import phonenumber_field.modelfields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0019_copy_mobile_phone_to_phone_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='useraddress',
            name='mobile_phone',
        ),
        migrations.AlterField(
            model_name='useraddress',
            name='phone',
            field=phonenumber_field.modelfields.PhoneNumberField(max_length=128, region=None, verbose_name='Phone Number'),
        ),
    ]
