# Generated by Django 5.2.9 on 2026-02-10 22:25

import core.fields.image
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('loyalty', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='loyaltytier',
            name='icon',
            field=core.fields.image.ImageAndSvgField(blank=True, null=True, upload_to='uploads/loyalty/', verbose_name='Icon'),
        ),
    ]
